package com.example.fuzzControll.service.impl;

import com.example.fuzzControll.conf.AflnetProperties;
import com.example.fuzzControll.constents.AlgorithmOneCmd;
import com.example.fuzzControll.constents.CmdConstent;
import com.example.fuzzControll.exception.AflnetException;
import com.example.fuzzControll.pojo.vo.AlgorithmOneCmdParams;
import com.example.fuzzControll.service.TestService;
import com.example.fuzzControll.tools.CmdTools;
import com.example.fuzzControll.tools.TestControlTools;
import com.jcraft.jsch.JSchException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

@Service("testService")
@Slf4j
public class TestServiceImpl implements TestService {

    CmdTools cmdTools = new CmdTools();

    @Autowired
    AflnetProperties aflnetProperties;


    @Override
    public void algorithmTwoStart(AlgorithmOneCmdParams algorithmOneCmdParams) {
        TestControlTools.setIsRunning(true);
        String aflnetCmd = getAlgorithmTwoFinalCmd(algorithmOneCmdParams);
        String pythonCmd = getAlgorithmTwoPythonCmd(algorithmOneCmdParams);
        if (aflnetCmd == null) {
            log.error("AlgorithmOne Cmd is null!");
            throw new AflnetException("Cmd Generate error!");
        }
        if (pythonCmd == null) {
            log.error("pythonCmd Cmd is null!");
            throw new AflnetException("Cmd Generate error!");
        }
        log.info("The cmd is [{}]", aflnetCmd);
        try {
            cmdTools.runProgramCmd(aflnetCmd,pythonCmd);
        } catch (JSchException e) {
            throw new RuntimeException(e);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }

    }


    private String getAlgorithmTwoPythonCmd(AlgorithmOneCmdParams algorithmOneCmdParams) {
        switch (algorithmOneCmdParams.getProtopcol().toLowerCase()) {
            case "dns":
                return "cd /home/lddc/SHENYANLONG/SMGfuzz/RL_Pytorch && python main_python.py --SUT dnsmasq";
            case "sip":
                return "cd /home/lddc/SHENYANLONG/SMGfuzz/RL_Pytorch && python main_python.py --SUT kamailio";
            case "ftp":
                return "cd /home/lddc/SHENYANLONG/SMGfuzz/RL_Pytorch && python main_python.py --SUT kamailio";
            case "rtsp":
                return "cd /home/lddc/SHENYANLONG/SMGfuzz/RL_Pytorch && python main_python.py --SUT live555";
            default:
                return null;
        }

    }

    private String getAlgorithmTwoFinalCmd(AlgorithmOneCmdParams algorithmOneCmdParams) {
        switch (algorithmOneCmdParams.getProtopcol().toLowerCase()) {
            case "dns":
                return dnsCmd(algorithmOneCmdParams);
            case "sip":
                return sipCmd(algorithmOneCmdParams);
            case "ftp":
                return ftpCmd(algorithmOneCmdParams);
            case "rtsp":
                return rtspCmd(algorithmOneCmdParams);
            default:
                return null;
            //todo 增加其他协议 的格式
        }
    }

    private String dnsCmd(AlgorithmOneCmdParams algorithmOneCmdParams) {
        String before = AlgorithmOneCmd.ALFNET_PATH + " -d " + AlgorithmOneCmd.ALFNET_INPUT_PATH + "/dnsmasq/in-dns "
                + AlgorithmOneCmd.ALFNET_OUTPUT_PATH + "/dnsmasq/src/output_dnsmasq_RLGfuzz";
        if (!"".equals(algorithmOneCmdParams.getTrainedModelInputPath())) {
            before = before + " -z " + algorithmOneCmdParams.getTrainedModelInputPath();
        }
        before = before + " -N " + algorithmOneCmdParams.getNetinfo() + " -m  none " + " -P " + algorithmOneCmdParams.getProtopcol() + " -D "
                + algorithmOneCmdParams.getWaiting();
        String after = " -q " + algorithmOneCmdParams.getStateSelectionAlgo() + " -s " + algorithmOneCmdParams.getSeedSelectionAlgo() + " -r '4' /home/lddc/SHENYANLONG/dnsmasq/src/dnsmasq";
        String finalCmd = before;
        if (algorithmOneCmdParams.getKill()) {
            finalCmd = finalCmd + " -K ";
        }
        if (algorithmOneCmdParams.getAwareMode())
            finalCmd = finalCmd + " -E ";
        if (algorithmOneCmdParams.getRegionMutationOperators())
            finalCmd = finalCmd + " -R ";
        if (!"".equals(algorithmOneCmdParams.getScript()))
            finalCmd = finalCmd + " -c " + algorithmOneCmdParams.getScript();
        finalCmd = finalCmd + after;
        return finalCmd;
//        return "/home/lddc/SHENYANLONG/SMGfuzz/afl-fuzz -d -i /home/lddc/SHENYANLONG/dnsmasq/in-dns -o /home/lddc/SHENYANLONG/dnsmasq/src/output_dnsmasq_RLGfuzz -z /home/lddc/SHENYANLONG/SMGfuzz/RL_Pytorch/Train_Result/dnsmasq/RL_Result -N udp://127.0.0.1/5353 -m none -P DNS -D 10000 -K -E -R -q 4 -s 4 -r '4' /home/lddc/SHENYANLONG/dnsmasq/src/dnsmasq";
    }

    private String sipCmd(AlgorithmOneCmdParams algorithmOneCmdParams) {
        String before = AlgorithmOneCmd.ALFNET_PATH + AlgorithmOneCmd.ALFNET_INPUT_PATH + "/kamailio/in-sip "
                + AlgorithmOneCmd.ALFNET_OUTPUT_PATH + "/kamailio/output_kamailio_RLGfuzz";
        if (!"".equals(algorithmOneCmdParams.getTrainedModelInputPath()))
            before = before + " -z " + algorithmOneCmdParams.getTrainedModelInputPath();
        before = before + " -N " + algorithmOneCmdParams.getNetinfo() + " -m  none -t 3000+ " + " -P " + algorithmOneCmdParams.getProtopcol() + " -l 5061 " + " -D "
                + algorithmOneCmdParams.getWaiting()+" -q " + algorithmOneCmdParams.getStateSelectionAlgo() + " -s " + algorithmOneCmdParams.getSeedSelectionAlgo();
        if (algorithmOneCmdParams.getKill()) {
            before = before + " -K ";
        }
        if (algorithmOneCmdParams.getAwareMode())
            before = before + " -E ";
        if (algorithmOneCmdParams.getRegionMutationOperators())
            before = before + " -R ";
        if (!"".equals(algorithmOneCmdParams.getScript()))
            before = before + " -c " + algorithmOneCmdParams.getScript();
        String after = " -d /home/lddc/SHENYANLONG/kamailio/src/kamailio -f /home/lddc/SHENYANLONG/kamailio/kamailio-basic.cfg -L /home/lddc/SHENYANLONG/kamailio/src/modules -n 1 -Y /home/lddc/SHENYANLONG/kamailio/runtime_dir";
        before = before + after;
        return before;
//        return "/home/lddc/SHENYANLONG/SMGfuzz/afl-fuzz -i /home/lddc/SHENYANLONG/kamailio/in-sip -o /home/lddc/SHENYANLONG/kamailio/output_kamailio_RLGfuzz -z /home/lddc/SHENYANLONG/SMGfuzz/RL_Pytorch/Train_Result/kamailio/RL_Result -N udp://127.0.0.1/5060 -m none -t 3000+ -P SIP -l 5061 -D 50000 -q 4 -s 4 -E -K -R -d -c /home/lddc/SHENYANLONG/kamailio/run_pjsip.sh /home/lddc/SHENYANLONG/kamailio/src/kamailio -f /home/lddc/SHENYANLONG/kamailio/kamailio-basic.cfg -L /home/lddc/SHENYANLONG/kamailio/src/modules -n 1 -Y /home/lddc/SHENYANLONG/kamailio/runtime_dir";
    }

    private String ftpCmd(AlgorithmOneCmdParams algorithmOneCmdParams) {
        String before = AlgorithmOneCmd.ALFNET_PATH  +" -m  none "+ AlgorithmOneCmd.ALFNET_INPUT_PATH + "/SMGfuzz/tutorials/lightftp/in-ftp "
                +AlgorithmOneCmd.ALFNET_OUTPUT_PATH+"/LightFTP/Source/Release/output_lightftp_RLGfuzz"+" -z "+algorithmOneCmdParams.getTrainedModelInputPath()
                +" -x /home/lddc/SHENYANLONG/SMGfuzz/tutorials/lightftp/ftp.dict"
                +" -N "+algorithmOneCmdParams.getNetinfo()+" -P "+algorithmOneCmdParams.getProtopcol()+" -D "
                +algorithmOneCmdParams.getWaiting()+" -d "+" -q "+algorithmOneCmdParams.getStateSelectionAlgo()+" -s "+algorithmOneCmdParams.getSeedSelectionAlgo();
        if(algorithmOneCmdParams.getKill()){
            before = before+" -K ";
        }
        if(algorithmOneCmdParams.getAwareMode())
            before = before+" -E ";
        if(algorithmOneCmdParams.getRegionMutationOperators())
            before = before+" -R ";
        if(!"".equals(algorithmOneCmdParams.getScript()))
            before = before+" -c "+algorithmOneCmdParams.getScript();
        before = before+ " /home/lddc/SHENYANLONG/LightFTP/Source/Release/fftp /home/lddc/SHENYANLONG/LightFTP/Source/Release/fftp.conf 2200";;
        return before;
//        return "/home/lddc/SHENYANLONG/SMGfuzz/afl-fuzz -m none -i /home/lddc/SHENYANLONG/SMGfuzz/tutorials/lightftp/in-ftp -o /home/lddc/SHENYANLONG/LightFTP/Source/Release/output_lightftp_RLGfuzz -N tcp://127.0.0.1/2200 -z /home/lddc/SHENYANLONG/SMGfuzz/RL_Pytorch/Train_Result/lightftp/RL_Result -x /home/lddc/SHENYANLONG/SMGfuzz/tutorials/lightftp/ftp.dict -P FTP -D 10000 -q 4 -s 4 -E -K -R -d -c /home/lddc/SHENYANLONG/LightFTP/Source/Release/ftpclean.sh /home/lddc/SHENYANLONG/LightFTP/Source/Release/fftp /home/lddc/SHENYANLONG/LightFTP/Source/Release/fftp.conf 2200";
    }

    private String rtspCmd(AlgorithmOneCmdParams algorithmOneCmdParams) {
        String before = AlgorithmOneCmd.ALFNET_PATH  + AlgorithmOneCmd.ALFNET_INPUT_PATH + "/SMGfuzz/tutorials/live555/in-rtsp "
                +AlgorithmOneCmd.ALFNET_OUTPUT_PATH+"/live555/testProgs/output_live555_RLGfuzz"+" -z "+algorithmOneCmdParams.getTrainedModelInputPath()
                +" -N "+algorithmOneCmdParams.getNetinfo()+" -x /home/lddc/SHENYANLONG/SMGfuzz/tutorials/live555/rtsp.dict "+" -P "+algorithmOneCmdParams.getProtopcol()+" -m none "+" -D "
                +algorithmOneCmdParams.getWaiting()+" -d -r 400";
        String after = " -q "+algorithmOneCmdParams.getStateSelectionAlgo()+" -s "+algorithmOneCmdParams.getSeedSelectionAlgo()
                +" /home/lddc/SHENYANLONG/live555/testProgs/testOnDemandRTSPServer 8554";
        if(algorithmOneCmdParams.getKill()){
            before = before+" -K ";
        }
        if(algorithmOneCmdParams.getAwareMode())
            before = before+" -E ";
        if(algorithmOneCmdParams.getRegionMutationOperators())
            before = before+" -R ";
        if(!"".equals(algorithmOneCmdParams.getScript()))
            before = before+" -c "+algorithmOneCmdParams.getScript();
        before = before+after;
        return before;
//        return "/home/lddc/SHENYANLONG/SMGfuzz/afl-fuzz -i /home/lddc/SHENYANLONG/SMGfuzz/tutorials/live555/in-rtsp -o /home/lddc/SHENYANLONG/live555/testProgs/output_live555_RLGfuzz -z /home/lddc/SHENYANLONG/SMGfuzz/RL_Pytorch/Train_Result/live555/RL_Result -N tcp://127.0.0.1/8554 -x /home/lddc/SHENYANLONG/SMGfuzz/tutorials/live555/rtsp.dict -m none -D 10000 -K -E -R -q 4 -s 4 -d /home/lddc/SHENYANLONG/live555/testProgs/testOnDemandRTSPServer 85540";
    }

    //todo 不同服务不同端口
    @Override
    public void algorithmOneStart(AlgorithmOneCmdParams algorithmOneCmdParams) {
        TestControlTools.setIsRunning(true);
        String cmd = cmdTools.parse(algorithmOneCmdParams);
        Date date = new Date();
        DateFormat df = new SimpleDateFormat("yyyy-MM-dd-hh-mm-ss-");
        String outputFileName = df.format(date) + algorithmOneCmdParams.getProtopcol() + "-output";
        String finalCmd = CmdConstent.RUN_AFLNET_BEFORE + outputFileName + CmdConstent.RUN_AFLNET_AFTER + cmd + aflnetProperties.getAflnetPath() + "live555/testProgs/testOnDemandRTSPServer 8554";
        log.info("The cmd is [{}]", finalCmd);
        try {
            cmdTools.runProgramCmd("source /etc/profile && " + finalCmd,"");
        } catch (JSchException e) {
            throw new RuntimeException(e);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }

    }

    @Override
    public void testStop() {
        TestControlTools.setIsRunning(false);
        log.info("Aflnet has been stopped ! ");
    }
}
